# @Copyright 2007 Kristjan Haule
import sys
from scipy import *
from scipy import linalg
from scipy import special
from scipy import misc
import transformations as tr

def ReducedWigner_spin(the):
    """ Order is :  up,dn
        [[d_{ 1/2,1/2}, d_{ 1/2,-1/2},
         [d_{-1/2,1/2}, d_{-1/2,-1/2}]]
    """
    dpp = cos(the/2)
    dpm = -sin(the/2)
    return array([[dpp,dpm],[-dpm,dpp]])

def ReducedWigner_p(the):
    """ Order is -1,0,1
       [d_{-1,-1}, d_{-1,0}, d_{-1,1}],
       [d_{ 0,-1}, d_{ 0,0}, d_{ 0,1}],
       [d_{ 1,-1}, d_{ 1,0}, d_{ 1,1}]
    """
    d11=(1+cos(the))/2.
    d10=-sin(the)/sqrt(2.)
    d1m = (1-cos(the))/2.
    d00 = cos(the)
    
    return array([[d11,-d10, d1m],
                  [d10, d00,-d10],
                  [d1m, d10, d11]])

def ReducedWigner_d(the):
    """ see: wikipedia.org/wiki/Wigner_D-matrix
        Order is -2,-1,0,1,2
        [d_{-2,-2}, d_{-2,-1}, d_{-2, 0}, d_{-2, 1}, d_{-2, 2}],
        [d_{-1,-2}, d_{-1,-1}, d_{-1, 0}, d_{-1, 1}, d_{-1, 2}],
        [d_{ 0,-2}, d_{ 0,-1}, d_{ 0, 0}, d_{ 0, 1}, d_{ 0, 2}],
        [d_{ 1,-2}, d_{ 1,-1}, d_{ 1, 0}, d_{ 1, 1}, d_{ 1, 2}],
        [d_{ 2,-2}, d_{ 2,-1}, d_{ 2, 0}, d_{ 2, 1}, d_{ 2, 2}]
    """
    d22 = 0.25*(1+cos(the))**2
    d21 = -0.5*sin(the)*(1.+cos(the))
    d20 = sqrt(3./8.)*sin(the)**2
    d2m1= -0.5*sin(the)*(1-cos(the))
    d2m2 = 0.25*(1-cos(the))**2
    d11  = 0.5*(2.*cos(the)**2+cos(the)-1.)
    d10  = -sqrt(3./8.)*sin(2*the)
    d1m1 = 0.5*(-2*cos(the)**2+cos(the)+1.)
    d00  = 0.5*(3*cos(the)**2-1.)

    return array([[d22,-d21, d20,-d2m1, d2m2],
                  [d21, d11,-d10, d1m1,-d2m1],
                  [d20, d10, d00, -d10,  d20],
                  [d2m1,d1m1,d10,  d11, -d21],
                  [d2m2,d2m1,d20,  d21,  d22]])

    
def ReducedWigner(the,j):
    if j==0.5:
        return ReducedWigner_spin(the)
    elif j==1:
        return ReducedWigner_p(the)
    elif j==2:
        return ReducedWigner_d(the)
    else:
        print 'Not implemented!'

def WignerMatrix(alpha,beta,gamma,j):
    dim = int(2*j+1)
    D = zeros((dim,dim),dtype=complex)
    dr = ReducedWigner(beta,j)
    if j==0.5:  # our convention is [-l:up, (-l+1):up, ....l:up, -l:dn,...l:up]
        mz = [0.5,-0.5]
    else:
        mz = range(-j,j+1)
    
    for mp in range(len(D)):
        for m in range(len(D)):
            D[mp,m] = exp(-mz[mp]*alpha*1j-mz[m]*gamma*1j)*dr[mp,m]
    return matrix(D)

def SpinRotation(R):
    Det = linalg.det(R)
    if (abs(abs(Det)-1)>1e-4): print 'Rotation does not have correct Determinant ', Det
    (al,be,ga) = tr.euler_from_matrix(array(R)*Det, 'rzyz')
    #print 'alpha,beta,gamma=', al, be, ga
    D = WignerMatrix(al,be,ga, 0.5)
    return D

def OrbitalRotation(R,l):
    Det = linalg.det(R)
    if (abs(abs(Det)-1)>1e-4): print 'Rotation does not have correct Determinant ', Det
    (al,be,ga) = tr.euler_from_matrix(array(R)*Det, 'rzyz')
    #print 'alpha,beta,gamma=', al, be, ga
    D = WignerMatrix(al,be,ga, l)
    if Det<0: D *= (-1)**l
    return D

def OrbitSpinRotation(R,l):
    D = OrbitalRotation(R,2)
    S = SpinRotation(R)
    Ds = vstack( (hstack( (S[0,0]*D, S[0,1]*D) ), hstack( (S[1,0]*D,S[1,1]*D) )) )
    return Ds





def mprint(Us):
    Qcomplex =  type(Us[0,0])==complex or type(Us[0,0])==complex128
    for i in range(shape(Us)[0]):
        for j in range(shape(Us)[1]):
            if Qcomplex:
                print "%11.8f %11.8f  " % (real(Us[i,j]), imag(Us[i,j])),
            else:
                print "%11.8f  " % Us[i,j],
        print
        
def StringToMatrix(cfstr,stype='complex'):
    mm=[]
    for line in cfstr.split('\n'):
        line = line.strip()
        if line:
            data = array(map(float,line.split()))
            if stype=='complex':
                mm.append( data[0::2]+data[1::2]*1j )
            else:
                mm.append( data )
                
    mm=matrix(mm)
    return mm


def test1():
    strBR1="""
     0.60623   0.00000   0.00000
     0.00000   0.60623   0.00000
     0.00000   0.00000   0.12885
    """
    startE="""
   -1.78323261     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000      -0.76307425     0.87260634       0.00000000     0.00000000       0.53542230     0.00031970       0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000   
    0.00000000     0.00000000      -2.46844237     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000       0.66411589    -0.00132445       0.00000000     0.00000000       0.00000000     0.00000000   
    0.00000000     0.00000000       0.00000000     0.00000000      -0.42722021     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000       0.66411589    -0.00132445       0.00000000     0.00000000   
    0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000      -1.94788809     0.00000000       0.00000000     0.00000000      -0.00608267    -0.00721856       0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000       0.53542230     0.00031970   
   -0.76307425    -0.87260634       0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000      -0.67614722     0.00000000       0.00000000     0.00000000      -0.00608267    -0.00721856       0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000   
    0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000      -0.00608267     0.00721856       0.00000000     0.00000000      -0.67614722     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000      -0.76307425     0.87260634   
    0.53542230    -0.00031970       0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000      -0.00608267     0.00721856       0.00000000     0.00000000      -1.94788809     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000   
    0.00000000     0.00000000       0.66411589     0.00132445       0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000      -0.42722021     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000   
    0.00000000     0.00000000       0.00000000     0.00000000       0.66411589     0.00132445       0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000      -2.46844237     0.00000000       0.00000000     0.00000000   
    0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000       0.53542230    -0.00031970       0.00000000     0.00000000      -0.76307425    -0.87260634       0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000      -1.78323261     0.00000000   
    """
    endE="""
   -1.78323266     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000      -0.08655796    -1.15595491       0.00000000     0.00000000       0.53542230     0.00031970       0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000   
    0.00000000     0.00000000      -2.46844241     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000       0.66411589    -0.00132445       0.00000000     0.00000000       0.00000000     0.00000000   
    0.00000000     0.00000000       0.00000000     0.00000000      -0.42722015     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000       0.66411589    -0.00132445       0.00000000     0.00000000   
    0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000      -1.94788813     0.00000000       0.00000000     0.00000000      -0.00087725     0.00939877       0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000       0.53542230     0.00031970   
   -0.08655796     1.15595491       0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000      -0.67614727     0.00000000       0.00000000     0.00000000      -0.00087725     0.00939877       0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000   
    0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000      -0.00087725    -0.00939877       0.00000000     0.00000000      -0.67614727     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000      -0.08655796    -1.15595491   
    0.53542230    -0.00031970       0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000      -0.00087725    -0.00939877       0.00000000     0.00000000      -1.94788813     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000   
    0.00000000     0.00000000       0.66411589     0.00132445       0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000      -0.42722015     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000   
    0.00000000     0.00000000       0.00000000     0.00000000       0.66411589     0.00132445       0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000      -2.46844241     0.00000000       0.00000000     0.00000000   
    0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000       0.53542230    -0.00031970       0.00000000     0.00000000      -0.08655796     1.15595491       0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000      -1.78323266     0.00000000   
    """
    strT2C="""
    0.00000000  0.00000000   -0.00000000  0.00000000   -0.00000000 -0.00000000    0.83205324 -0.00000000   -0.00000000 -0.00000000    0.21593690 -0.32743698    0.00000000 -0.00000000   -0.00000000 -0.00000000   -0.00000000  0.00000000   -0.21593690  0.32743698
   -0.21593690 -0.32743698    0.00000000 -0.00000000   -0.00000000  0.00000000   -0.00000000 -0.00000000    0.21593690  0.32743698    0.00000000  0.00000000    0.83205323 -0.00000000   -0.00000000  0.00000000   -0.00000000 -0.00000000   -0.00000000 -0.00000000
   -0.00000000  0.00000000    0.00000000 -0.00000000    0.00000000  0.00000000   -0.00000000  0.00000000    0.00000000 -0.00000000   -0.00000000 -0.00000000    0.00000000  0.00000000    0.00000000  0.00000000    1.00000001  0.00000000    0.00000000  0.00000000
    0.00000000  0.00000000    1.00000000  0.00000000    0.00000000  0.00000000    0.00000000  0.00000000   -0.00000000 -0.00000000   -0.00000000  0.00000000    0.00000000  0.00000000    0.00000000  0.00000000   -0.00000000 -0.00000000    0.00000000 -0.00000000
    0.00000000 -0.00000000    0.00000000  0.00000000   -0.00000000  0.00000000   -0.30538089 -0.46306582   -0.00000000  0.00000000    0.58835049  0.00000000    0.00000000 -0.00000000   -0.00000000  0.00000000    0.00000000 -0.00000000   -0.58835049 -0.00000000
   -0.58835048  0.00000000   -0.00000000  0.00000000    0.00000000  0.00000000    0.00000000  0.00000000    0.58835048  0.00000000   -0.00000000 -0.00000000   -0.30538089  0.46306582    0.00000000  0.00000000    0.00000000  0.00000000    0.00000000  0.00000000
    0.00000000  0.00000000    0.00000000  0.00000000    1.00000000  0.00000000    0.00000000  0.00000000    0.00000000  0.00000000    0.00000000  0.00000000    0.00000000  0.00000000    0.00000000  0.00000000    0.00000000  0.00000000    0.00000000  0.00000000
    0.00000000  0.00000000    0.00000000  0.00000000    0.00000000  0.00000000    0.00000000  0.00000000    0.00000000  0.00000000    0.00000000  0.00000000    0.00000000  0.00000000    1.00000000  0.00000000    0.00000000  0.00000000    0.00000000  0.00000000
    0.70710679  0.00000000    0.00000000  0.00000000    0.00000000  0.00000000    0.00000000  0.00000000    0.70710679  0.00000000    0.00000000  0.00000000    0.00000000  0.00000000    0.00000000  0.00000000    0.00000000  0.00000000    0.00000000  0.00000000
    0.00000000  0.00000000    0.00000000  0.00000000    0.00000000  0.00000000    0.00000000  0.00000000    0.00000000  0.00000000    0.70710679  0.00000000    0.00000000  0.00000000    0.00000000  0.00000000    0.00000000  0.00000000    0.70710679  0.00000000
    """
    strEorb="""
   -1.89955387     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000      -0.08847416     0.44862172       0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000       1.00842224    -0.09013870   
    0.00000000     0.00000000      -1.89955386     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000      -0.08847416    -0.44862172       0.00000000     0.00000000       0.00000000     0.00000000       1.00842224     0.09013870       0.00000000     0.00000000   
    0.00000000     0.00000000       0.00000000     0.00000000      -2.46844241     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000       0.66411589     0.00132445       0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000   
    0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000      -2.46844241     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000       0.66411589    -0.00132445       0.00000000     0.00000000       0.00000000     0.00000000   
   -0.08847416    -0.44862172       0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000      -1.21237646     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000       0.33532475    -0.77896215   
    0.00000000     0.00000000      -0.08847416     0.44862172       0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000      -1.21237646     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000       0.33532475     0.77896215       0.00000000     0.00000000   
    0.00000000     0.00000000       0.00000000     0.00000000       0.66411589    -0.00132445       0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000      -0.42722015     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000   
    0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000       0.66411589     0.00132445       0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000      -0.42722015     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000   
    0.00000000     0.00000000       1.00842224    -0.09013870       0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000       0.33532475    -0.77896215       0.00000000     0.00000000       0.00000000     0.00000000      -1.30838869     0.00000000       0.00000000     0.00000000   
    1.00842224     0.09013870       0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000       0.33532475     0.77896215       0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000      -1.30838869     0.00000000   

    """
    BR1 = StringToMatrix(strBR1,'float')
    E0 = StringToMatrix(startE)
    Ef = StringToMatrix(endE)
    CF = StringToMatrix(strT2C)
    Eorb = StringToMatrix(strEorb)
    
    
    R=matrix([[-0.553932,  -0.832562,  0.000000],
              [0.832562,  -0.553932,  0.000000],
              [0.000000,   0.000000,  1.000000]])

    Ds = OrbitSpinRotation(R,2)
    Ec = Ds * E0 * Ds.H
    Eo = conj(CF) * Ec * CF.T

    
    print 'Ds='
    mprint(Ds)
    print 'Efinal='
    mprint( Ec )
    print
    print 'Efinal_fort='
    mprint( Ef )
    print
    print 'Diff='
    mprint( Ec-Ef)
    print sum(abs(Ec-Ef))
    print 'Eorbital='
    mprint(Eo)
    print 'Diff='
    mprint( Eo-Eorb)
    print sum(abs(Eo-Eorb))

def test2():
    strBR1="""
     0.60623   0.00000   0.00000 
     0.00000   0.60623   0.00000 
     0.00000   0.00000   0.12885 
    """
    startE0="""
   -1.78323261     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000      -0.76307425     0.87260634       0.00000000     0.00000000       0.53542230     0.00031970       0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000   
    0.00000000     0.00000000      -2.46844237     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000       0.66411589    -0.00132445       0.00000000     0.00000000       0.00000000     0.00000000   
    0.00000000     0.00000000       0.00000000     0.00000000      -0.42722021     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000       0.66411589    -0.00132445       0.00000000     0.00000000   
    0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000      -1.94788809     0.00000000       0.00000000     0.00000000      -0.00608267    -0.00721856       0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000       0.53542230     0.00031970   
   -0.76307425    -0.87260634       0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000      -0.67614722     0.00000000       0.00000000     0.00000000      -0.00608267    -0.00721856       0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000   
    0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000      -0.00608267     0.00721856       0.00000000     0.00000000      -0.67614722     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000      -0.76307425     0.87260634   
    0.53542230    -0.00031970       0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000      -0.00608267     0.00721856       0.00000000     0.00000000      -1.94788809     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000   
    0.00000000     0.00000000       0.66411589     0.00132445       0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000      -0.42722021     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000   
    0.00000000     0.00000000       0.00000000     0.00000000       0.66411589     0.00132445       0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000      -2.46844237     0.00000000       0.00000000     0.00000000   
    0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000       0.53542230    -0.00031970       0.00000000     0.00000000      -0.76307425    -0.87260634       0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000      -1.78323261     0.00000000   
    """
    startE1="""
   -1.20483925     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000      -0.51797459     0.59232494       0.00000000     0.00000000       0.41315240     0.00024669       0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000   
    0.00000000     0.00000000      -2.16984560     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000       0.62265402    -0.00124176       0.00000000     0.00000000       0.00000000     0.00000000   
    0.00000000     0.00000000       0.00000000     0.00000000      -0.42722021     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000       0.62265402    -0.00124176       0.00000000     0.00000000   
    0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000      -1.71660572     0.00000000       0.00000000     0.00000000      -0.00471551    -0.00559609       0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000       0.41315240     0.00024669   
   -0.51797459    -0.59232494       0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000      -0.46110887     0.00000000       0.00000000     0.00000000      -0.00471551    -0.00559609       0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000   
    0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000      -0.00471551     0.00559609       0.00000000     0.00000000      -0.46110887     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000      -0.51797459     0.59232494   
    0.41315240    -0.00024669       0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000      -0.00471551     0.00559609       0.00000000     0.00000000      -1.71660572     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000   
    0.00000000     0.00000000       0.62265402     0.00124176       0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000      -0.42722021     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000   
    0.00000000     0.00000000       0.00000000     0.00000000       0.62265402     0.00124176       0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000      -2.16984560     0.00000000       0.00000000     0.00000000   
    0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000       0.41315240    -0.00024669       0.00000000     0.00000000      -0.51797459    -0.59232494       0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000      -1.20483925     0.00000000   
    """
    startE2="""
   -1.43329482     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000      -0.61347334    -0.70153164       0.00000000     0.00000000       0.45062545    -0.00026907       0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000   
    0.00000000     0.00000000      -2.16982483     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000       0.62265103     0.00124175       0.00000000     0.00000000       0.00000000     0.00000000   
    0.00000000     0.00000000       0.00000000     0.00000000      -0.42722021     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000       0.62265103     0.00124175       0.00000000     0.00000000   
    0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000      -1.71662330     0.00000000       0.00000000     0.00000000      -0.00512053     0.00607675       0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000       0.45062545    -0.00026907   
   -0.61347334     0.70153164       0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000      -0.54371527     0.00000000       0.00000000     0.00000000      -0.00512053     0.00607675       0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000   
    0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000      -0.00512053    -0.00607675       0.00000000     0.00000000      -0.54371527     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000      -0.61347334    -0.70153164   
    0.45062545     0.00026907       0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000      -0.00512053    -0.00607675       0.00000000     0.00000000      -1.71662330     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000   
    0.00000000     0.00000000       0.62265103    -0.00124175       0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000      -0.42722021     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000   
    0.00000000     0.00000000       0.00000000     0.00000000       0.62265103    -0.00124175       0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000      -2.16982483     0.00000000       0.00000000     0.00000000   
    0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000       0.45062545     0.00026907       0.00000000     0.00000000      -0.61347334     0.70153164       0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000      -1.43329482     0.00000000   
    """
    startE3="""
   -1.56145363     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000      -0.66757684    -0.76340118       0.00000000     0.00000000       0.47034063    -0.00028084       0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000   
    0.00000000     0.00000000      -2.16982483     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000       0.62265103     0.00124175       0.00000000     0.00000000       0.00000000     0.00000000   
    0.00000000     0.00000000       0.00000000     0.00000000      -0.42722021     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000       0.62265103     0.00124175       0.00000000     0.00000000   
    0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000      -1.71662330     0.00000000       0.00000000     0.00000000      -0.00533856     0.00633549       0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000       0.47034063    -0.00028084   
   -0.66757684     0.76340118       0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000      -0.59100221     0.00000000       0.00000000     0.00000000      -0.00533856     0.00633549       0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000   
    0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000      -0.00533856    -0.00633549       0.00000000     0.00000000      -0.59100221     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000      -0.66757684    -0.76340118   
    0.47034063     0.00028084       0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000      -0.00533856    -0.00633549       0.00000000     0.00000000      -1.71662330     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000   
    0.00000000     0.00000000       0.62265103    -0.00124175       0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000      -0.42722021     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000   
    0.00000000     0.00000000       0.00000000     0.00000000       0.62265103    -0.00124175       0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000      -2.16982483     0.00000000       0.00000000     0.00000000   
    0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000       0.47034063     0.00028084       0.00000000     0.00000000      -0.66757684     0.76340118       0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000      -1.56145363     0.00000000   
    """
    endE="""
   -1.78323266     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000      -0.08655796    -1.15595491       0.00000000     0.00000000       0.53542230     0.00031970       0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000   
    0.00000000     0.00000000      -2.46844241     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000       0.66411589    -0.00132445       0.00000000     0.00000000       0.00000000     0.00000000   
    0.00000000     0.00000000       0.00000000     0.00000000      -0.42722015     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000       0.66411589    -0.00132445       0.00000000     0.00000000   
    0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000      -1.94788813     0.00000000       0.00000000     0.00000000      -0.00087725     0.00939877       0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000       0.53542230     0.00031970   
   -0.08655796     1.15595491       0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000      -0.67614727     0.00000000       0.00000000     0.00000000      -0.00087725     0.00939877       0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000   
    0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000      -0.00087725    -0.00939877       0.00000000     0.00000000      -0.67614727     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000      -0.08655796    -1.15595491   
    0.53542230    -0.00031970       0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000      -0.00087725    -0.00939877       0.00000000     0.00000000      -1.94788813     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000   
    0.00000000     0.00000000       0.66411589     0.00132445       0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000      -0.42722015     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000   
    0.00000000     0.00000000       0.00000000     0.00000000       0.66411589     0.00132445       0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000      -2.46844241     0.00000000       0.00000000     0.00000000   
    0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000       0.53542230    -0.00031970       0.00000000     0.00000000      -0.08655796     1.15595491       0.00000000     0.00000000       0.00000000     0.00000000       0.00000000     0.00000000      -1.78323266     0.00000000   
    """
    BR1 = StringToMatrix(strBR1,'float')
    E0 = StringToMatrix(startE0)
    E1 = StringToMatrix(startE1)
    E2 = StringToMatrix(startE2)
    E3 = StringToMatrix(startE3)
    
    rotij = zeros((4,3,3),dtype=float)
    rotij[0,:,:] = [[ 1.0, 0.0, 0.0],
                    [ 0.0, 1.0, 0.0],
                    [ 0.0, 0.0, 1.0]]
    rotij[1,:,:] = [[ 0.0,-1.0, 0.0],
                    [ 1.0, 0.0, 0.0],
                    [ 0.0, 0.0, 1.0]]
    rotij[2,:,:] = [[-1.0, 0.0, 0.0],
                    [ 0.0, 1.0, 0.0],
                    [ 0.0, 0.0,-1.0]]
    rotij[3,:,:]=  [[ 0.0, 1.0, 0.0],
                    [ 1.0, 0.0, 0.0],
                    [ 0.0, 0.0,-1.0]]


    for i in range(4):
        rot = BR1 * matrix(rotij[i,:,:]) * BR1.I
        print 'rotij_1='
        mprint(rotij[i,:,:])
        print 'rotij_2='
        mprint(rot)
    
    rot = BR1 * matrix(rotij[1,:,:]) * BR1.I
    Ds = OrbitSpinRotation(rot,2)
    print 'D='
    mprint(Ds)
    E1n = Ds * E1 * Ds.H

    rot = BR1 * rotij[2,:,:] * BR1.I
    Ds = OrbitSpinRotation(rot,2)
    E2n = Ds * E2 * Ds.H
    
    rot = BR1 * rotij[3,:,:] * BR1.I
    Ds = OrbitSpinRotation(rot,2)
    E3n = Ds * E3 * Ds.H
    

    print 'E0='
    mprint( E0 )
    print
    print 'E1n='
    mprint( E1n )
    print
    print sum(abs(E0-E1n))
    print
    print 'E2n='
    mprint( E2n )
    print
    print sum(abs(E0-E2n))
    print
    print 'E3n='
    mprint( E3n )
    print
    print sum(abs(E0-E3n))
    

    
if __name__ == '__main__':
    test1()
    test2()
